/***************************************************************************
    NWNX Clock - Clock functions plugin
    Copyright (C) 2006 Ingmar Stieger (Papillon, papillon@nwnx.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************/

/*

	---------------------------------------------------------------------------------------
	Date		Name		Reason
	---------------------------------------------------------------------------------------
	010107		KFW			Initial.
	081107		CarterDC	Refactored to NWNX4 1.08.
	081807		KFW			Syntax check.
	081807		KFW			Mapsize fix to NWNX4Clock module.
    101208      KFW         Compiled for NWNX4 1.09 and NWN2 1.13.
	---------------------------------------------------------------------------------------

*/

#include "clock.h"

/***************************************************************************
    NWNX and DLL specific functions
***************************************************************************/

Clock * plugin;

DLLEXPORT Plugin * GetPluginPointerV2( ){
	return( plugin );
}


BOOL APIENTRY DllMain(HMODULE hModule, DWORD ul_reason_for_call, LPVOID lpReserved){

	if(			ul_reason_for_call == DLL_PROCESS_ATTACH ){
		plugin = new Clock( );
		TCHAR szPath[ MAX_PATH ];
		GetModuleFileName( hModule, szPath, MAX_PATH );
		plugin->SetPluginFullPath( szPath );
	}
	else if(	ul_reason_for_call == DLL_PROCESS_DETACH )
		delete plugin;

    return( TRUE );

}


/***************************************************************************
    Implementation of Clock Plugin
***************************************************************************/

Clock :: Clock( ){

	header		=	"NWNX Clock Plugin V.";
	header	   +=	CLOCK_VERSION;
	header	   +=	"\n(c) 2006 by Ingmar Stieger (Papillon)\nvisit us at http://www.nwnx.org\n";

	description	=	"This plugin gets the system's date and time.";

	subClass	=	"CLOCK";

	version		=	CLOCK_VERSION;

	return;

}


Clock :: ~Clock( ){

	wxLogMessage( wxT( "* Plugin unloaded." ) );

}


bool Clock :: Init( TCHAR * nwnxhome ){

	assert( GetPluginFileName( ) );

	/* Log file */
	wxString logfile( nwnxhome ); 
	logfile.append( wxT( "\\" ) );
	logfile.append( GetPluginFileName( ) );
	logfile.append( wxT( ".txt" ) );
	logger = new wxLogNWNX( logfile, wxString( header.c_str( ) ) );

	wxLogMessage( wxT( "* Plugin initialized." ) );
	return( TRUE );

}


VOID Clock :: GetFunctionClass( TCHAR * fClass ){

	_tcsncpy_s( fClass, 128, wxT( "CLOCK" ), 5 ); 

	return;

}


VOID Clock :: SetString( PCHAR psFunction, PCHAR psParam1, INT nParam2, PCHAR psValue ){
	
	wxLogTrace(TRACE_VERBOSE, wxT("* Plugin GetString(0x%x, %s, %d)"), 0x0, psParam1, nParam2);

	wxString wxRequest( psFunction, wxConvUTF8 );
	wxString function( psFunction, wxConvUTF8 );
	wxString timerName( psParam1, wxConvUTF8 );

	// Log the NWNX4 string for debugging purposes.
	wxLogMessage(
		wxT( "* Plugin SetString( psFunction = %s, psParam1 = %s, nParam2 = %d, psValue = %s )" ),
		psFunction,
		psParam1,
		nParam2,
		psValue );

	
	// Check for a function string.
	if(			function == wxT( "" ) ){
		wxLogMessage( wxT( "* Function not specified." ) );
		return;
	}

	return;

}


PCHAR Clock :: GetString( PCHAR psFunction, PCHAR psParam1, INT nParam2 ){

	// Log the NWNX4 string for debugging purposes.
	wxLogMessage(
		wxT( "* Plugin GetString( psFunction = %s, psParam1 = %s, nParam2 = %d ) " ),
		psFunction,
		psParam1,
		nParam2 );

	// Variables.
	wxString wxRequest( psFunction, wxConvUTF8 );
	wxString function( psFunction, wxConvUTF8 );
	wxString parameter1( psParam1, wxConvUTF8 );


	// Check for a function string.
	if(			function == wxT( "" ) ){
		wxLogMessage( wxT( "* Function not specified." ) );
		return( NULL );
	}

	else if(	function == wxT( "DATE" ) )			GetDate( );
	else if(	function == wxT( "TIME" ) )			GetTime( );
	else if(	function == wxT( "UNIXTIME" ) )		GetUNIXTime( );

	// Name, Description, Subclass, Version query.
	else{

		// Process generic functions.
		wxString query = ProcessQueryFunction( function.c_str( ) );
		if( query != wxT( "" ) )
			sprintf_s( returnBuffer, MAX_BUFFER, "%s", query );
		else{
			wxLogMessage( wxT( "* Unknown function '%s' called." ), function );
			return( NULL );
		}
	
	}

	return( returnBuffer );

}


// This method refreshes the class' date and time information.
INT Clock :: UpdateClock( ){

	// Get the system's date and time info.
	GetLocalTime( &SystemTime );

	if( SystemTime.wYear != NULL ){
		wxLogMessage( "* NWNX4/CLOCK : System date and time acquired successfully." );
		return( TRUE );
	}
	else{
		wxLogMessage( "* NWNX4/CLOCK : System date and time acquisition failed!" );
		return( FALSE );
	}

}


// This method returns the date in the format (mm/dd/yyyy) back to the NWN2 game.
VOID Clock :: GetDate( ){

	// Variables.
	CHAR szDate[ 11 ]		= "";

	// Get the latest clock info.
	if( !UpdateClock( ) )
		return;
	
	// Form the date string.
	StringCbPrintf( szDate, 11, "%02hu/%02hu/%04hu", SystemTime.wMonth, SystemTime.wDay, SystemTime.wYear );
	// Log precise data input/output.
	wxLogMessage( "Clock :: GetDate( ) = %s", szDate );
	// Pass it back to the NWN2 game.
	WriteNWN2String( szDate );

	return;

}


// This method returns the time in the format (hh:mm:ss) back to the NWN2 game.
VOID Clock :: GetTime( ){

	// Variables.
	CHAR szTime[ 9 ]		= "";
	
	// Get the latest clock info.
	if( !UpdateClock( ) )
		return;

	// Form the time string.
	StringCbPrintf( szTime, 9, "%02hu:%02hu:%02hu", SystemTime.wHour, SystemTime.wMinute, SystemTime.wSecond );
	// Log precise data input/output.
	wxLogMessage( "Clock :: GetTime( ) = %s", szTime );
	// Pass it back to the NWN2 game.
	WriteNWN2String( szTime );
	
	return;

}


// This method returns the time since in seconds since Jan 1st 1970.
VOID Clock :: GetUNIXTime( ){

	// Variables.
	HMODULE hNTDLL			= NULL;
	CHAR szTime[ 33 ]		= "";
	FILETIME FileTime;
	LARGE_INTEGER liTime;
	DWORD dwTime			= 0;
	pRtlTimeToSecondsSince1970 RtlTimeToSecondsSince1970;


	// Get the latest clock info.
	if( !UpdateClock( ) )
		return;

	// Convert the temporary clock info to file time.
	if( SystemTimeToFileTime( &SystemTime, &FileTime ) == NULL ){
		wxLogMessage( "* NWNX4/CLOCK : GetUNIXTime( ) error. SystemTimeToFileTime( SYSTEMTIME ) failed!" );
		return;
	}

	// Initialize a large integer with the file time.
	liTime.LowPart = FileTime.dwLowDateTime;
	liTime.HighPart = FileTime.dwHighDateTime;

	// Link to NTDLL.Dll.
	if( ( hNTDLL = LoadLibrary( "ntdll.dll" ) ) == NULL ){
		wxLogMessage( "* NWNX4/CLOCK : GetUNIXTime( ) error. LoadLibrary( NTDLL ) failed!" );
		return;
	}

	// Setup pointer to function NTDLL.RtlTimeToSecondsSince1970.
	if( ( RtlTimeToSecondsSince1970 = ( pRtlTimeToSecondsSince1970 ) GetProcAddress( hNTDLL, "RtlTimeToSecondsSince1970" ) ) == NULL ){
		wxLogMessage( "* NWNX4/CLOCK : GetUNIXTime( ) error. GetProcAddress( RtlTimeToSecondsSince1970 ) failed!" );
		return;
	}

	// Invoke RtlTimeToSecondsSince1970( ).
	if( !RtlTimeToSecondsSince1970( &liTime, &dwTime ) ){
		wxLogMessage( "* NWNX4/CLOCK : GetUNIXTime( ) error. RtlTimeToSecondsSince1970( ) failed!" );
		return;
	}

	// Form the time string.
	StringCbPrintf( szTime, 33, "%lu", dwTime );
	// Log precise data input/output.
	wxLogMessage( "GetUNIXTime( ) = %lu", dwTime );
	// Pass it back to the NWN2 game.
	WriteNWN2String( szTime );

	return;

}


// This method will write text to NWN2's NWNXGetString(..) return value.
VOID Clock :: WriteNWN2String( PCHAR pszNWN2String ){

	// Copy the desired string into NWN2's one.
	nwnxcpy( returnBuffer, pszNWN2String );

	return;

}